package edu.nyu.sps.fling;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;

import java.util.Locale;

public class MainActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        final TextView textView = (TextView)findViewById(R.id.textView);

        final GestureDetector gestureDetector = new GestureDetector(this, new GestureDetector.SimpleOnGestureListener() {

            private String[] arrow = {    //Unicode arrow symbols
                    "\u2192",    //right
                    "\u2197",    //upper right
                    "\u2191",    //up
                    "\u2196",    //upper left
                    "\u2190",    //left
                    "\u2199",    //lower left
                    "\u2193",    //down
                    "\u2198"    //lower right
            };

            @Override
            public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
                float distanceX = e2.getX() - e1.getX();
                float distanceY = e2.getY() - e1.getY();

                //3 o'clock is 0 degrees, 12 o'clock is 90 degrees, 6 o'clock is -90 degrees.
                float theta = (float)Math.toDegrees(Math.atan2(-distanceY, distanceX));

                //theta is in the range
                //-180 <= theta <= 180

                float t = theta;
                if (t < 0f) {
                    t += 360f;    //means t = t + 360f;
                }

                //t is now in the range
                //0 <= t <= 360

                t *= arrow.length / 360f;    //means t = t * arrow.length / 360f;

                //t is now in the range
                //0 <= t <= arrow.length

                int subscript = Math.round(t);    //to nearest integer
                if (subscript == arrow.length) {
                    subscript = 0;
                }

                //subscript is now in the range
                //0 <= subscript < arrow.length

                String text = String.format(Locale.US,
                                  "Horizontal distance: %g pixels\n"
                                + "Vertical distance: %g pixels\n"
                                + "Diagonal distance: %g pixels\n"
                                + "Elapsed time: %d milliseconds\n"
                                + "Horizontal velocity: %g pixels/second\n"
                                + "Vertical velocity: %g pixels/second\n"
                                + "Diagonal velocity: %g pixels/second\n"
                                + "Angle: %.1f\u00B0 counterclockwise from 3 o\u2019clock\n"
                                + "Direction: %s\n\n"
                                + "MotionEvent 1:\n"
                                + "%s\n"
                                + "MotionEvent 2:\n"
                                + "%s",

                        distanceX, distanceY,
                        Math.hypot(distanceX, distanceY),
                        e2.getEventTime() - e1.getEventTime(),
                        velocityX, velocityY,
                        Math.hypot(velocityX, velocityY),
                        theta,
                        arrow[subscript],
                        description(e1), description(e2)
                );

                textView.setGravity(Gravity.TOP | Gravity.START); //In English, START means LEFT
                textView.setText(text);
                return true;
            }

            private String description(MotionEvent motionEvent) {
                int action = motionEvent.getAction();
                return String.format(Locale.US,
                        "Action: %s\n"
                                + "Pointer count: %d\n"
                                + "Event time: %d milliseconds since boot\n"
                                + "Location: (%g, %g)\n"
                                + "Pressure: %g\n"
                                + "Size: %g\n",

                        action == MotionEvent.ACTION_DOWN ? "ACTION_DOWN" :
                                action == MotionEvent.ACTION_UP ? "ACTION_UP" : "unknown",

                        motionEvent.getPointerCount(),
                        motionEvent.getEventTime(),
                        motionEvent.getX(), motionEvent.getY(),
                        motionEvent.getPressure(),
                        motionEvent.getSize()
                );
            }
        });

        textView.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                gestureDetector.onTouchEvent(event);
                return true;
            }
        });
    }
}
